<?php

class OpMonClusterManager
{

    private $mapper;

    private $oids = array(
        "processes"            => ".1.3.6.1.2.1.25.4.2.1.2",
        "storage"              => ".1.3.6.1.2.1.25.2.3.1.3",
        "ifName"               => ".1.3.6.1.2.1.31.1.1.1.1",
        "ifSpeed"              => ".1.3.6.1.2.1.2.2.1.5",
        "ifDescr"              => ".1.3.6.1.2.1.2.2.1.2",
        "ifAlias"              => ".1.3.6.1.2.1.31.1.1.1.18",
    );

    public function __construct()
    {
        $this->mapper = new \Models\WebServices\Mapper\Mapper();
        include '/usr/local/opmon/etc/config.php';
    }

    private function getCommunity($data)
    {
        $snmpCommunity = new SNMPCommunity($data->globalauthid);
        return  new SNMPIntegration($snmpCommunity, $data->globalhostaddress . ":" . $data->globalport);
    }

    public function getStorage($data)
    {
        $data->oid = $this->oids['storage'];
        $snmpIntegration = $this->getCommunity($data);
        $scanResult = $snmpIntegration->bulkwalk($data->oid);
        $result = array();

        foreach ($scanResult as $key => $value) {
            $obj = new StdClass();
            $obj->value = $obj->key = $value;
            $result[] = $obj;
            $this->mapper->addElement(
                $value,
                $value,
                array(
                    'storages' => $value
                )
            );
        }

        $this->mapper->setComboData($result);
        return $this->mapper->getOutput();
    }

    public function getProcess($data)
    {
        $data->oid = $this->oids['processes'];
        $snmpIntegration = $this->getCommunity($data);
        $scanResult = $snmpIntegration->bulkwalk($data->oid);
        $result = array();

        foreach ($scanResult as $key => $value) {
            $value = preg_replace('/(["])/', ${1}, $value);
            $obj = new StdClass();
            $obj->value = $obj->key = $value;
            $result[] = $obj;
            $this->mapper->addElement(
                "Process_{$value}",
                $value,
                array(
                    'process' => $value
                )
            );
        }

        $this->mapper->setComboData($result);
        return $this->mapper->getOutput();
    }

    public function getInterfaces($data)
    {
        $data->oid = ($data->localmethod) ? $this->oids[$data->localmethod] : '.1.3.6.1.2.1.31.1.1.1.1';
        $snmpIntegration = $this->getCommunity($data);
        $scanResult = $snmpIntegration->bulkwalk($data->oid);
        $result = array();

        foreach ($scanResult as $key => $value) {
            $obj = new StdClass();
            $obj->value = $obj->key = $value;
            $result[] = $obj;
            $this->mapper->addElement(
                "Inteface_{$value}",
                $value,
                array(
                    'interface' => $value,
                    'ifSpeed' => ($this->getIfSpeed($key, $snmpIntegration))?:0
                )
            );
        }

        $this->mapper->setComboData($result);
        return $this->mapper->getOutput();
    }

    public function getIfSpeed($oid, $snmpIntegration)
    {
        preg_match("/^.+\.(\d+)$/", $oid, $matches);

        if (count($matches) != 2) {
            return false;
        }

        $if_index = $matches[1];
        $oid = $this->oids['ifSpeed'] . ".$if_index";
        $ifSpeed =  $snmpIntegration->bulkwalk($oid);
        return $ifSpeed[$oid] /= 1000;
    }

    public function updateCommand($version)
    {
    }
}
